#!/usr/bin/env scheme-script
;; -*- mode: scheme; coding: utf-8 -*- !#
;; Copyright © 2018-2019 Göran Weinholt <goran@weinholt.se>
;; SPDX-License-Identifier: MIT
#!r6rs

(import
  (rnrs (6))
  (srfi :64 testing)
  (text-mode terminfo)
  (text-mode console events))

(define (get-output expr p1 p2)
  ;; Get the output from an expression as a string
  (utf8->string
   (call-with-bytevector-output-port
     (lambda (p)
       (let ((test (terminfo-procedure expr (make-termstate #f p 38400))))
         (test p1 p2))))))

(define (byte-value expr p1 p2)
  ;; Takes an expression and translates its pushed value to a byte
  (- (bytevector-u8-ref
      (call-with-bytevector-output-port
        (lambda (p)
          (let ((test (terminfo-procedure (string-append expr "%'a'%+%c")
                                          (make-termstate #f p 38400))))
            (test p1 p2))))
      0)
     (char->integer #\a)))

(test-begin "terminfo-operators")

(test-equal (byte-value "%p1%p2%+" 1 2) 3)
(test-equal (byte-value "%p1%p2%-" 1 2) -1)
(test-equal (byte-value "%p1%p2%*" 1 2) 2)
(test-equal (byte-value "%p1%p2%/" 10 2) 5)
(test-equal (byte-value "%p1%p2%m" 10 3) 1)

(test-equal (byte-value "%p1%p2%&" 1 3) 1)
(test-equal (byte-value "%p1%p2%|" 1 2) 3)
(test-equal (byte-value "%p1%p2%^" 1 3) 2)

(test-equal (byte-value "%p1%p2%A" 0 0) 0)
(test-equal (byte-value "%p1%p2%A" 0 1) 0)
(test-equal (byte-value "%p1%p2%A" 1 0) 0)
(test-equal (byte-value "%p1%p2%A" 1 1) 1)

(test-equal (byte-value "%p1%p2%O" 0 0) 0)
(test-equal (byte-value "%p1%p2%O" 0 1) 1)
(test-equal (byte-value "%p1%p2%O" 1 0) 1)
(test-equal (byte-value "%p1%p2%O" 1 1) 1)

(test-equal (byte-value "%p1%p2%!" 0 0) 1)
(test-equal (byte-value "%p1%p2%!" 0 1) 0)

(test-equal (byte-value "%p1%p2%~" 0 0) -1)
(test-equal (byte-value "%p1%p2%~" 0 1) -2)
(test-equal (byte-value "%p1%p2%~" 0 2) -3)
(test-equal (byte-value "%p1%p2%~" 0 -1) 0)
(test-equal (byte-value "%p1%p2%~" 0 -2) 1)
(test-equal (byte-value "%p1%p2%~" 0 -3) 2)

(test-equal (byte-value "%p1%p2%>" 0 1) 0)
(test-equal (byte-value "%p1%p2%>" 1 0) 1)
(test-equal (byte-value "%p1%p2%<" 0 1) 1)
(test-equal (byte-value "%p1%p2%<" 1 0) 0)

(test-equal (byte-value "%'a'" 0 0) (char->integer #\a))
(test-equal (byte-value "%'b'" 0 0) (char->integer #\b))

(test-equal (byte-value "%{1}" 0 0) 1)
(test-equal (byte-value "%{75}" 0 0) 75)
(test-end)

(test-begin "terminfo-printf")
(test-equal (get-output "%p1%d" 65 0) "65")

(test-equal (get-output "\x1B;[%p1%d;0;0;0q%p2%:-16.16s" 10 "hello")
            "\x1B;[10;0;0;0qhello           ")
(test-equal (get-output "%p1%:16.16s" "hello" 0)
            "           hello")
(test-equal (get-output "%p1%:-16s" "hello" 0)
            "hello           ")
(test-equal (get-output "%p1%4.2X" #xa 0) "  0A")
(test-equal (get-output "%p1%4.2x" #xa 0) "  0a")
(test-equal (get-output "%p1%4.2d" 1 0)   "  01")
(test-equal (get-output "%p1%4.2o" 10 0)  "  12")
(test-equal (get-output "%p1%:-4.2X" #xa 0) "0A  ")
(test-equal (get-output "%p1%:-4.2x" #xa 0) "0a  ")
(test-equal (get-output "%p1%:-4.2d" 1 0)   "01  ")
(test-equal (get-output "%p1%:-4.2o" 10 0)  "12  ")

(define test-data
  '(("%:-2d"     -16 "-16")
    ("%:-2d"      -1 "-1")
    ("%:-2d"       0 "0 ")
    ("%:-2d"       1 "1 ")
    ("%:-2d"       8 "8 ")
    ("%:-2d"      10 "10")
    ("%:-2d"      16 "16")
    ("%:-2d"     123 "123")
    ("%:-2d"    1234 "1234")
    ("%:-2d"   12345 "12345")
    ("%:-2d"  123456 "123456")
    ("%02X"     -16 "FFFFFFF0")
    ("%02X"      -1 "FFFFFFFF")
    ("%02X"       0 "00")
    ("%02X"       1 "01")
    ("%02X"       8 "08")
    ("%02X"      10 "0A")
    ("%02X"      16 "10")
    ("%02X"     123 "7B")
    ("%02X"    1234 "4D2")
    ("%02X"   12345 "3039")
    ("%02X"  123456 "1E240")
    ("%02d"     -16 "-16")
    ("%02d"      -1 "-1")
    ("%02d"       0 "00")
    ("%02d"       1 "01")
    ("%02d"       8 "08")
    ("%02d"      10 "10")
    ("%02d"      16 "16")
    ("%02d"     123 "123")
    ("%02d"    1234 "1234")
    ("%02d"   12345 "12345")
    ("%02d"  123456 "123456")
    ("%02x"     -16 "fffffff0")
    ("%02x"      -1 "ffffffff")
    ("%02x"       0 "00")
    ("%02x"       1 "01")
    ("%02x"       8 "08")
    ("%02x"      10 "0a")
    ("%02x"      16 "10")
    ("%02x"     123 "7b")
    ("%02x"    1234 "4d2")
    ("%02x"   12345 "3039")
    ("%02x"  123456 "1e240")
    ("%03d"     -16 "-16")
    ("%03d"      -1 "-01")
    ("%03d"       0 "000")
    ("%03d"       1 "001")
    ("%03d"       8 "008")
    ("%03d"      10 "010")
    ("%03d"      16 "016")
    ("%03d"     123 "123")
    ("%03d"    1234 "1234")
    ("%03d"   12345 "12345")
    ("%03d"  123456 "123456")
    ("%1d"     -16 "-16")
    ("%1d"      -1 "-1")
    ("%1d"       0 "0")
    ("%1d"       1 "1")
    ("%1d"       8 "8")
    ("%1d"      10 "10")
    ("%1d"      16 "16")
    ("%1d"     123 "123")
    ("%1d"    1234 "1234")
    ("%1d"   12345 "12345")
    ("%1d"  123456 "123456")
    ("%2.2X"     -16 "FFFFFFF0")
    ("%2.2X"      -1 "FFFFFFFF")
    ("%2.2X"       0 "00")
    ("%2.2X"       1 "01")
    ("%2.2X"       8 "08")
    ("%2.2X"      10 "0A")
    ("%2.2X"      16 "10")
    ("%2.2X"     123 "7B")
    ("%2.2X"    1234 "4D2")
    ("%2.2X"   12345 "3039")
    ("%2.2X"  123456 "1E240")
    ("%2.2d"     -16 "-16")
    ("%2.2d"      -1 "-01")
    ("%2.2d"       0 "00")
    ("%2.2d"       1 "01")
    ("%2.2d"       8 "08")
    ("%2.2d"      10 "10")
    ("%2.2d"      16 "16")
    ("%2.2d"     123 "123")
    ("%2.2d"    1234 "1234")
    ("%2.2d"   12345 "12345")
    ("%2.2d"  123456 "123456")
    ("%2d"     -16 "-16")
    ("%2d"      -1 "-1")
    ("%2d"       0 " 0")
    ("%2d"       1 " 1")
    ("%2d"       8 " 8")
    ("%2d"      10 "10")
    ("%2d"      16 "16")
    ("%2d"     123 "123")
    ("%2d"    1234 "1234")
    ("%2d"   12345 "12345")
    ("%2d"  123456 "123456")
    ("%3d"     -16 "-16")
    ("%3d"      -1 " -1")
    ("%3d"       0 "  0")
    ("%3d"       1 "  1")
    ("%3d"       8 "  8")
    ("%3d"      10 " 10")
    ("%3d"      16 " 16")
    ("%3d"     123 "123")
    ("%3d"    1234 "1234")
    ("%3d"   12345 "12345")
    ("%3d"  123456 "123456")
    ("%4.4X"     -16 "FFFFFFF0")
    ("%4.4X"      -1 "FFFFFFFF")
    ("%4.4X"       0 "0000")
    ("%4.4X"       1 "0001")
    ("%4.4X"       8 "0008")
    ("%4.4X"      10 "000A")
    ("%4.4X"      16 "0010")
    ("%4.4X"     123 "007B")
    ("%4.4X"    1234 "04D2")
    ("%4.4X"   12345 "3039")
    ("%4.4X"  123456 "1E240")
    ("%X"     -16 "FFFFFFF0")
    ("%X"      -1 "FFFFFFFF")
    ("%X"       0 "0")
    ("%X"       1 "1")
    ("%X"       8 "8")
    ("%X"      10 "A")
    ("%X"      16 "10")
    ("%X"     123 "7B")
    ("%X"    1234 "4D2")
    ("%X"   12345 "3039")
    ("%X"  123456 "1E240")
    ("%d"     -16 "-16")
    ("%d"      -1 "-1")
    ("%d"       0 "0")
    ("%d"       1 "1")
    ("%d"       8 "8")
    ("%d"      10 "10")
    ("%d"      16 "16")
    ("%d"     123 "123")
    ("%d"    1234 "1234")
    ("%d"   12345 "12345")
    ("%d"  123456 "123456")
    ("%x"     -16 "fffffff0")
    ("%x"      -1 "ffffffff")
    ("%x"       0 "0")
    ("%x"       1 "1")
    ("%x"       8 "8")
    ("%x"      10 "a")
    ("%x"      16 "10")
    ("%x"     123 "7b")
    ("%x"    1234 "4d2")
    ("%x"   12345 "3039")
    ("%x"  123456 "1e240")
    ("%o"     -16 "37777777760")
    ("%o"      -1 "37777777777")
    ("%o"       0 "0")
    ("%o"       1 "1")
    ("%o"       8 "10")
    ("%o"      10 "12")
    ("%o"      16 "20")
    ("%o"     123 "173")
    ("%o"    1234 "2322")
    ("%o"   12345 "30071")
    ("%o"  123456 "361100")
    ("%:-03d"     -16 "-16")
    ("%:-03d"      -1 "-1 ")
    ("%:-03d"       0 "0  ")
    ("%:-03d"       1 "1  ")
    ("%:-03d"       8 "8  ")
    ("%:-03d"      10 "10 ")
    ("%:-03d"      16 "16 ")
    ("%:-03d"     123 "123")
    ("%:-03d"    1234 "1234")
    ("%:-03d"   12345 "12345")
    ("%:-03d"  123456 "123456")
    ("%03d"     -16 "-16")
    ("%03d"      -1 "-01")
    ("%03d"       0 "000")
    ("%03d"       1 "001")
    ("%03d"       8 "008")
    ("%03d"      10 "010")
    ("%03d"      16 "016")
    ("%03d"     123 "123")
    ("%03d"    1234 "1234")
    ("%03d"   12345 "12345")
    ("%03d"  123456 "123456")
    ("%3d"     -16 "-16")
    ("%3d"      -1 " -1")
    ("%3d"       0 "  0")
    ("%3d"       1 "  1")
    ("%3d"       8 "  8")
    ("%3d"      10 " 10")
    ("%3d"      16 " 16")
    ("%3d"     123 "123")
    ("%3d"    1234 "1234")
    ("%3d"   12345 "12345")
    ("%3d"  123456 "123456")
    ("% 03d"     -16 "-16")
    ("% 03d"      -1 "-01")
    ("% 03d"       0 " 00")
    ("% 03d"       1 " 01")
    ("% 03d"       8 " 08")
    ("% 03d"      10 " 10")
    ("% 03d"      16 " 16")
    ("% 03d"     123 " 123")
    ("% 03d"    1234 " 1234")
    ("% 03d"   12345 " 12345")
    ("% 03d"  123456 " 123456")
    ("%4.x"     -16 "fffffff0")
    ("%4.x"      -1 "ffffffff")
    ("%4.x"       0 "    ")
    ("%4.x"       1 "   1")
    ("%4.x"       8 "   8")
    ("%4.x"      10 "   a")
    ("%4.x"      16 "  10")
    ("%4.x"     123 "  7b")
    ("%4.x"    1234 " 4d2")
    ("%4.x"   12345 "3039")
    ("%4.x"  123456 "1e240")
    ("% 4x"     -16 "fffffff0")
    ("% 4x"      -1 "ffffffff")
    ("% 4x"       0 "   0")
    ("% 4x"       1 "   1")
    ("% 4x"       8 "   8")
    ("% 4x"      10 "   a")
    ("% 4x"      16 "  10")
    ("% 4x"     123 "  7b")
    ("% 4x"    1234 " 4d2")
    ("% 4x"   12345 "3039")
    ("% 4x"  123456 "1e240")
    ("%.x"     -16 "fffffff0")
    ("%.x"      -1 "ffffffff")
    ("%.x"       0 "")
    ("%.x"       1 "1")
    ("%.x"       8 "8")
    ("%.x"      10 "a")
    ("%.x"      16 "10")
    ("%.x"     123 "7b")
    ("%.x"    1234 "4d2")
    ("%.x"   12345 "3039")
    ("%.x"  123456 "1e240")
    ("%#4.4x"     -16 "0xfffffff0")
    ("%#4.4x"      -1 "0xffffffff")
    ("%#4.4x"       0 "0000")
    ("%#4.4x"       1 "0x0001")
    ("%#4.4x"       8 "0x0008")
    ("%#4.4x"      10 "0x000a")
    ("%#4.4x"      16 "0x0010")
    ("%#4.4x"     123 "0x007b")
    ("%#4.4x"    1234 "0x04d2")
    ("%#4.4x"   12345 "0x3039")
    ("%#4.4x"  123456 "0x1e240")
    ("%#4.4d"     -16 "-0016")
    ("%#4.4d"      -1 "-0001")
    ("%#4.4d"       0 "0000")
    ("%#4.4d"       1 "0001")
    ("%#4.4d"       8 "0008")
    ("%#4.4d"      10 "0010")
    ("%#4.4d"      16 "0016")
    ("%#4.4d"     123 "0123")
    ("%#4.4d"    1234 "1234")
    ("%#4.4d"   12345 "12345")
    ("%#4.4d"  123456 "123456")
    ("%#4.4o"     -16 "037777777760")
    ("%#4.4o"      -1 "037777777777")
    ("%#4.4o"       0 "0000")
    ("%#4.4o"       1 "0001")
    ("%#4.4o"       8 "0010")
    ("%#4.4o"      10 "0012")
    ("%#4.4o"      16 "0020")
    ("%#4.4o"     123 "0173")
    ("%#4.4o"    1234 "02322")
    ("%#4.4o"   12345 "030071")
    ("%#4.4o"  123456 "0361100")
    ("%#4.4X"     -16 "0XFFFFFFF0")
    ("%#4.4X"      -1 "0XFFFFFFFF")
    ("%#4.4X"       0 "0000")
    ("%#4.4X"       1 "0X0001")
    ("%#4.4X"       8 "0X0008")
    ("%#4.4X"      10 "0X000A")
    ("%#4.4X"      16 "0X0010")
    ("%#4.4X"     123 "0X007B")
    ("%#4.4X"    1234 "0X04D2")
    ("%#4.4X"   12345 "0X3039")
    ("%#4.4X"  123456 "0X1E240")
    ("%:#4x"     -16 "0xfffffff0")
    ("%:#4x"      -1 "0xffffffff")
    ("%:#4x"       0 "   0")
    ("%:#4x"       1 " 0x1")
    ("%:#4x"       8 " 0x8")
    ("%:#4x"      10 " 0xa")
    ("%:#4x"      16 "0x10")
    ("%:#4x"     123 "0x7b")
    ("%:#4x"    1234 "0x4d2")
    ("%:#4x"   12345 "0x3039")
    ("%:#4x"  123456 "0x1e240")
    ("%4.4c"     123 "{")
    ("%4.4c"   12345 "9")
    ("%4.4c"  123456 "@")
    ("%1.1i%p1%d"     -16 "-15")
    ("%1.1i%p1%d"      -1 "0")
    ("%1.1i%p1%d"       0 "1")
    ("%1.1i%p1%d"       1 "2")
    ("%1.1i%p1%d"       8 "9")
    ("%1.1i%p1%d"      10 "11")
    ("%1.1i%p1%d"      16 "17")
    ("%1.1i%p1%d"     123 "124")
    ("%1.1i%p1%d"    1234 "1235")
    ("%1.1i%p1%d"   12345 "12346")
    ("%1.1i%p1%d"  123456 "123457")
    ("%6.0d"     -16 "   -16")
    ("%6.0d"      -1 "    -1")
    ("%6.0d"       0 "      ")
    ("%6.0d"       1 "     1")
    ("%6.0d"       8 "     8")
    ("%6.0d"      10 "    10")
    ("%6.0d"      16 "    16")
    ("%6.0d"     123 "   123")
    ("%6.0d"    1234 "  1234")
    ("%6.0d"   12345 " 12345")
    ("%6.0d"  123456 "123456")
    ("%063d"     -16 "-00000000000000000000000000000000000000000000000000000000000016")
    ("%063d"      -1 "-00000000000000000000000000000000000000000000000000000000000001")
    ("%063d"       0 "000000000000000000000000000000000000000000000000000000000000000")
    ("%063d"       1 "000000000000000000000000000000000000000000000000000000000000001")
    ("%063d"       8 "000000000000000000000000000000000000000000000000000000000000008")
    ("%063d"      10 "000000000000000000000000000000000000000000000000000000000000010")
    ("%063d"      16 "000000000000000000000000000000000000000000000000000000000000016")
    ("%063d"     123 "000000000000000000000000000000000000000000000000000000000000123")
    ("%063d"    1234 "000000000000000000000000000000000000000000000000000000000001234")
    ("%063d"   12345 "000000000000000000000000000000000000000000000000000000000012345")
    ("%063d"  123456 "000000000000000000000000000000000000000000000000000000000123456")
    ("% 063d"     -16 "-00000000000000000000000000000000000000000000000000000000000016")
    ("% 063d"      -1 "-00000000000000000000000000000000000000000000000000000000000001")
    ("% 063d"       0 " 00000000000000000000000000000000000000000000000000000000000000")
    ("% 063d"       1 " 00000000000000000000000000000000000000000000000000000000000001")
    ("% 063d"       8 " 00000000000000000000000000000000000000000000000000000000000008")
    ("% 063d"      10 " 00000000000000000000000000000000000000000000000000000000000010")
    ("% 063d"      16 " 00000000000000000000000000000000000000000000000000000000000016")
    ("% 063d"     123 " 00000000000000000000000000000000000000000000000000000000000123")
    ("% 063d"    1234 " 00000000000000000000000000000000000000000000000000000000001234")
    ("% 063d"   12345 " 00000000000000000000000000000000000000000000000000000000012345")
    ("% 063d"  123456 " 00000000000000000000000000000000000000000000000000000000123456")))

(test-assert
 (for-all (lambda (x) x)
          (map (lambda (test)
                 (let ((output
                        (get-output (car test) (cadr test) 0))
                       (expected (caddr test)))
                   (unless (equal? output expected)
                     (display "FAIL: ")
                     (write `(get-output ,(car test) ,(cadr test) 0))
                     (display "\nOutput: ")
                     (write output)
                     (display "\nExpect: ")
                     (write expected)
                     (newline))
                   (equal? output expected)))
               test-data)))

(test-end)

(test-begin "terminfo-compat")
(test-equal (get-output "\x1B;[%i%d;%dR" 10 20) "\x1B;[21;11R") ;bad example
(test-end)

(test-begin "terminfo-misc")
(test-equal (call-with-bytevector-output-port
              (lambda (p)
                ((terminfo-procedure "%{229}%u" (make-termstate #f p 38400)))))
            (string->utf8 "å"))

(test-equal (get-output "\x1B;a%p1%?%p1%'_'%>%t\x1;%'`'%-%;%' '%+%c%p2%?%p2%'_'%>%t\x1;%'`'%-%;%' '%+%c"
                        10 20)
            "\x1B;a*4")

(test-equal (get-output "\x1B;&f0a%p1%dk0d%p2%l%dL%p2%s" 42 "hello")
            "\x1b;&f0a42k0d5Lhello")
(test-end)

(test-begin "terminfo-sexpr")
(test-equal (terminfo-db->sexpr (get-terminfo-db "dumb"))
            '("dumb|80-column dumb tty"
              (am . #t)
              (cols . 80)
              (bel . "\a")
              (cr . "\r")
              (cud1 . "\n")
              (ind . "\n")))
(test-equal (terminfo-db->sexpr
             (sexpr->terminfo-db
              (terminfo-db->sexpr (get-terminfo-db "ansi"))))
            (terminfo-db->sexpr (get-terminfo-db "ansi")))
(test-end)


(define (mktermstate db)
  (let-values ([(out _extract) (open-bytevector-output-port)])
    (let ((in (open-bytevector-input-port #vu8())))
      (let ((state (make-termstate #f #f 38400)))
        (termstate-ti-set! state db)
        state))))

(define (check-input keydefs input)
  (let-values ([(partial? key code)
                (termstate-translate-input (mktermstate (sexpr->terminfo-db
                                                         `("test" ,@keydefs)))
                                           (string->utf8 input))])
    (list partial? key (utf8->string code))))

(test-begin "terminfo-decode-utf8")
(test-equal (check-input '() "x") '(#f #\x "x"))
(test-equal (check-input '() "ö") '(#f #\ö "ö"))
(test-equal (check-input '() "\x32FF;") '(#f #\x32FF "\x32FF;"))
(test-equal (check-input '() "\x11029;") '(#f #\x11029 "\x11029;"))
(test-end)

(test-begin "terminfo-decode-user6")
(define user6/7 '((u6 . "\x1b;[%i%d;%dR") (u7 . "\x1B;[6n")))
(test-assert (not (car (check-input user6/7 "\x1b;[1;1R")))) ;not partial
(test-assert (cursor-report-event? (cadr (check-input user6/7 "\x1b;[1;1R"))))
(test-end)

(test-begin "terminfo-decode-user8")
(define user8/9 '((u8 . "\x1b;[?1;2c") (u9 . "\x1B;[c")))
(test-assert (not (car (check-input user8/9 "\x1b;[?1;2c")))) ;not partial
(test-assert (device-attributes-event? (cadr (check-input user8/9 "\x1b;[?1;2c"))))
(test-end)

(test-begin "terminfo-reject-unknown")
(test-assert (unknown-event? (cadr (check-input '() "\x1b;[12^"))))
(test-end)

(test-begin "terminfo-decode-keys")

(test-equal (check-input '() "\t") '(#f KEY_TAB "\t"))
(test-equal (check-input '() "\r") '(#f KEY_ENTER "\r"))
(test-equal (check-input '() "\a") '(#f (ctrl . #\g) "\a"))
(test-equal (check-input '() "\x1b;\a") '(#f (alt . (ctrl . #\g)) "\x1b;\a"))
(test-equal (check-input '() "\x1b;a") '(#f (alt . #\a) "\x1b;a"))

(test-equal (check-input '((kf1 . "\x1b;[11~")) "\x1b;[11~")
            '(#f KEY_F1 "\x1b;[11~"))

(test-equal (check-input '((kf1 . "\x1b;[11~")) "\x1b;\x1b;[11~")
            '(#f (alt . KEY_F1) "\x1b;\x1b;[11~"))

(test-equal (check-input '((kf1 . "\x1b;[11~")) "\x1b;[11~x")
            '(#f KEY_F1 "\x1b;[11~"))

(test-end)

(exit (if (zero? (test-runner-fail-count (test-runner-get))) 0 1))
